#!/usr/bin/env python
# -*- Encoding: latin-1 -*-
"""
A class for handling franctions
written 2004 by Marek Kubica

(Will use) Uses code from Mike Hostetler's fraction.py, but is more powerful
and is _not_ an extension of fraction.py, but uses some concepts of those.
Licensed under LGPL

Differences between mh and mk fract:
- mk does not work (will be fixed)
- mk cannot reduce (will also be fixed)
- mk can work with whole numbers
- mk version is more documented
- mk has unitttests, and is in big parts test-first developed
- the performance was not yet tested (but has to be done)

TODO:
- make more unit tests
- let fraction work with simple operations
- implement reducing

Changelog:
0.0.6 - namespace mangling (means semi-private internal functions)
0.0.5 - __repr__, __float__, __int__ hooks, more unittests
0.0.4 - Made all written unittests pass"""

__version__ = '0.0.5'
__author__ = 'Marek Kubica'

class Fraction(object):
    """This class is intended to handle fractions, to operate with them
    yust like with normal numbers"""
    
    #__slots__ = ['whole', 'upper', 'lower', 'reduce']
    
    def __init__(self, whole=0, upper=0, lower=1, reduce=False):
        """The constructor of the Fraction
        this method is overloaded (well, kind of):
        it can be called by
        Fraction(whole=1, upper=1, lower=2)
        or by
        Fraction("a b/c")
        (well Fraction(whole="a b/c") would also work but is illogical)
        """
        if type(whole) == type(str()) or type(whole) == type(unicode()):
            # alternate constructor - 'a b/c' - string constructor
            self.input(whole)
        else:
            # traditional constructor - eg. whole=1, upper=1, lower=2
            self.__setWhole(whole)
            self.__setUpper(upper)
            self.__setLower(lower)
        
        self.setReduce(reduce)

    def __repr__(self):
	"""The 'print' representation of the fraction.
	This is also the hook for the repr() & str() builtin.
	Uses self.getValue() internally."""
	return self.getValue()

    def __float__(self):
	"""Returns the fraction as float.
	This is the hook for the float() builtin."""
        # well, this *could* be made shorter, but now it's logical and easy
        
        # calculate the fraction part
        return_float = float(self.getUpper()) / float(self.getLower())
        # add the whole part
        return_float = return_float + self.getWhole()
        # return the result
        return return_float

    def __int__(self):
	"""Returns just the whole part of the fraction.
	This is the hook for the int() builtin."""
	return self.getWhole()
    
    def input(self, fraction):
        """Input strings in the form 'a b/c'"""
        # test the format of the number
        #+or should the set* functions care?
        #+ > probably the set* functions
        
        fraction = fraction.split()
        whole = int(fraction[0])
        self.__setWhole(whole)
	try:
    	    fract_part = fraction[1].split('/')
    	    upper = int(fract_part[0])
    	    lower = int(fract_part[1])
    	    self.__setUpper(upper)
    	    self.__setLower(lower)
	except IndexError:
	    # we just go a whole value, so we set the fraction part to zero
	    self.__setUpper(0)
	    self.__setLower(1)

    def __getPrettyprint(self):
	"""The mk prettyprint mechanism"""
	w, u, l = self.getWhole(), self.getUpper(), self.getLower()
	
	if float(u) / float(l) != 0:
	    # get the lenghts of the numbers
	    lw, lu, ll = len(str(w)), len(str(u)), len(str(l))
	    
	    # create the line
	    if lu > ll:
		line = lu * '-'
	    else:
		line = ll * '-'
	    
	    # create the prettyprinted lines
	    pline1 = (lw + 1) * ' ' + str(u)
	    pline2 = str(w) + ' ' + line
	    pline3 = (lw + 1) * ' ' + str(l)
	    
	    # make them to one prettyprinted multiline string
	    pprint = pline1 + '\n' + pline2 + '\n' + pline3
	    # return the string
	    return pprint
	else:
	    # we have no fraction part, so just return the whole part
	    return str(w)

    
    def prettyprint(self):
	"""A small prettyprint wrapper"""
	print self.__getPrettyprint()
        
    def getWhole(self):
        """Returns the whole part of the fraction"""
        return self.whole
    
    def getUpper(self):
	"""Returns the demominator of a fraction"""
        return self.upper
    
    def getLower(self):
	"""Returns the numerator"""
        return self.lower
    
    def __setWhole(self, number):
        """Set the 'whole' value.
	This function does checking and if the value is bad raises ValueError."""
        if type(number) == type(int()) or type(number) == type(float()):
            self.whole = number
        else:
            raise ValeError('A number is expected')
            
    def __setUpper(self, number):
        try:
            int(number)
        except (ValueError, TypeError):
            raise ValueError('A number is expected')
        else:
            self.upper = number
    
    def setReduce(self, state):
        """Set the state of reduce
	Well, we probably need some tweaking here.
	Checking the values is neccessary"""
        #print state
        self.reduce = state
    
    def getReduce(self):
	"""Returns the current state of the 'reduce' switch."""
        return self.reduce
    
    def reduce(self):
	"""Does actually run reduceing.
	Should this take care of the reduce switch? I think not!
	However this is not yet implemented, maybe I can use code from
	the mh fraction implementation"""
        raise NotImplementedError

    
    def __setLower(self, number):
        """Sets the value of Lower.
	This function checks the value before setting any bad and raises a nice ValueError."""
        # should we also accept string numbers? No! (isn't it?)
        
        # check if input _is_ a number
        try:
            int(number)
        except ValueError, TypeError:
            raise ValueError('A number is expected')
        
        # okay, this is a number, check if the number is zero
        if number == 0:
            raise ValueError('A zero value is not allowed')
        else:
            self.lower = number
    
    def getValue(self):
        """Returns the simple string representation of the fraction.
	If the fraction part is zero it returns just the whole part."""
        w, u, l = self.getWhole(), self.getUpper(), self.getLower()
	
	if float(u) / float(l) != 0.0:
    	    string_output = str(w) + ' ' + str(u) + '/' + str(l)
    	    return string_output
	else:
	    string_output = str(w)
	    return string_output